#define _CRT_SECURE_NO_WARNINGS

#include <stdlib.h>
#include <stdio.h>
#include <conio.h>
#include <memory.h>

#include "bad_aon_intro_seq.h"
#include "bad_aon_end_seq.h"

unsigned char out_data[1024*1024];

int out_ptr=0;

unsigned char displayState[4][9][8];
unsigned char displayStatePrev[4][9][8];


enum {
	SEG_A=0,
	SEG_B,
	SEG_C,
	SEG_D,
	SEG_E,
	SEG_F,
	SEG_G,
	SEG_DP
};

#define DSA	0x01
#define DSB	0x02
#define DSC	0x04
#define DSD	0x08
#define DSE	0x10
#define DSF	0x20
#define DSG	0x40
#define DSH	0x80


struct {
	unsigned char* data;
	int wdt;
	int hgt;
	int rle;
	int palette_offset;
	int raster_offset;
	bool flip;
	int bpp;
} BMP;



#define CODE_AREA_SIZE	0x800

int control_ptr;
int control_bank;
int control_frame;


void control_inc(int n)
{
	for(int i=0;i<n;++i)
	{
		++control_ptr;

		if((control_ptr&0xff00)==0x2000) control_ptr+=0x2000; else
		if((control_ptr&0xff00)==0x6000) control_ptr+=0x2000; else
		if((control_ptr&0xff00)==0xa000) control_ptr+=0x2000; else

			if((control_ptr&0xff00)==0xe000)
			{
				control_ptr=CODE_AREA_SIZE;
				++control_bank;

				printf("bank change at frame %i\n",control_frame);
			}

	}
}



bool load_bmp(char* filename)
{
	FILE* file=fopen(filename,"rb");

	if(!file) return false;

	if(BMP.data)
	{
		free(BMP.data);
		BMP.data=NULL;
	}

	fseek(file,0,SEEK_END);
	int size=ftell(file);
	fseek(file,0,SEEK_SET);

	BMP.data=(unsigned char*)malloc(size);
	fread(BMP.data,size,1,file);
	fclose(file);

	if(BMP.data[0]!='B'||BMP.data[1]!='M')
	{
		printf("ERR: Not BMP file format\n");
		exit(1);
	}

	BMP.raster_offset=BMP.data[10]+(BMP.data[11]<<8)+(BMP.data[12]<<16)+(BMP.data[13]<<24);
	BMP.palette_offset=14+BMP.data[14]+(BMP.data[15]<<8)+(BMP.data[16]<<16)+(BMP.data[17]<<24);
	BMP.wdt=BMP.data[18]+(BMP.data[19]<<8)+(BMP.data[20]<<16)+(BMP.data[21]<<24);
	BMP.hgt=BMP.data[22]+(BMP.data[23]<<8)+(BMP.data[24]<<16)+(BMP.data[25]<<24);
	BMP.bpp=BMP.data[28]+(BMP.data[29]<<8);
	BMP.rle=BMP.data[30]+(BMP.data[31]<<8)+(BMP.data[32]<<16)+(BMP.data[33]<<24);

	if(BMP.rle!=0||BMP.bpp!=24)
	{
		printf("ERR: Only 24-bit BMP supported\n");
		exit(1);
	}

	BMP.flip=(BMP.hgt>0)?false:true;

	if(BMP.flip) BMP.hgt=-BMP.hgt;

	return true;
}



int get_pixel(int sx,int sy)
{
	if(!BMP.data) return 0;

	return BMP.data[BMP.raster_offset+((BMP.hgt-1-sy)*BMP.wdt+sx)*3]>>6;
}



int pitch_table[256];

int pitch_table_len;

unsigned char music_frames[120*5*60];

int music_frames_all;
int music_frames_ptr;



bool parse_music_text(const char* filename)
{
	char line[1024];
	char* end;

	memset(pitch_table,0,sizeof(pitch_table));
	memset(music_frames,0,sizeof(music_frames));

	pitch_table_len=0;
	music_frames_all=0;
	music_frames_ptr=0;

	pitch_table[pitch_table_len++]=0;	//silence always the first entry

	FILE* fin=fopen(filename,"rt");

	if(!fin) return false;

	fgets(line,sizeof(line),fin);

	while(fgets(line,sizeof(line),fin))
	{
		int time=(int)strtod(&line[0],&end);

		int period=(int)strtod(end+1,NULL);

		if(period) period=1000000/(1193181/period);	//convert to 1mhz

		int pitch_id=-1;

		for(int i=0;i<pitch_table_len;++i)
		{
			if(pitch_table[i]==period)
			{
				pitch_id=i;
				break;
			}
		}

		if(pitch_id<0)
		{
			pitch_id=pitch_table_len;
			pitch_table[pitch_table_len++]=period;
		}

		if(pitch_table_len>=256)
		{
			printf("too many pitches!\n");
			break;
		}

		for(int i=0;i<time;++i)
		{
			music_frames[music_frames_all++]=pitch_id;
		}
	}

	printf("%i pitches, %i frames\n",pitch_table_len,music_frames_all);

	fclose(fin);

	return true;
}



int fetch_sound_frame(void)
{
	if(music_frames_ptr>=music_frames_all)
	{
		return 0;
	}

	return music_frames[music_frames_ptr++];
}


int pitch_changes=0;
int pitch_writes=0;
int pitch_id1_prev=-1;
int pitch_id2_prev=-1;

bool convert_frame(int frame,int led_part,bool skip)
{
	char filename[1024];

	sprintf(filename,"../testconv/%4.4u.bmp",frame);

	if(!load_bmp(filename)) return false;

	//printf("%s\n",filename);

	//convert all displays at once

	int sy=0;

	for(int dp=0;dp<4;++dp)
	{
		int sx=0;

		for(int dg=0;dg<9;++dg)
		{
			displayState[dp][dg][SEG_A]=get_pixel(sx+1,sy+0);
			displayState[dp][dg][SEG_B]=get_pixel(sx+2,sy+1);
			displayState[dp][dg][SEG_C]=get_pixel(sx+2,sy+3);
			displayState[dp][dg][SEG_D]=get_pixel(sx+1,sy+4);
			displayState[dp][dg][SEG_E]=get_pixel(sx+0,sy+3);
			displayState[dp][dg][SEG_F]=get_pixel(sx+0,sy+1);
			displayState[dp][dg][SEG_G]=get_pixel(sx+1,sy+2);
			displayState[dp][dg][SEG_DP]=get_pixel(sx+3,sy+5);
			//displayState[dp][(frame/8/4)%9][(frame/4)&7]=3;
			sx+=4;
		}

		sy+=6;	
	}

	//pack frame

	int mask=0;

	int pitch_id1=fetch_sound_frame();
	int pitch_id2=fetch_sound_frame();

	if(pitch_id1_prev!=pitch_id1)
	{
		pitch_id1_prev=pitch_id1;
		mask|=0x4000;
		++pitch_changes;
	}

	if(pitch_id2_prev!=pitch_id2)
	{
		pitch_id2_prev=pitch_id2;
		mask|=0x2000;
		++pitch_changes;
	}
	//printf("%2.2x %2.2x ",pitch_id1,pitch_id2);

	for(int dg=0;dg<9;++dg)
	{
		unsigned char b1=0;
		unsigned char b2=0;

		int diff_cnt=0;

		for(int sg=0;sg<8;++sg)
		{
			if(displayState[led_part][dg][sg]!=displayStatePrev[led_part][dg][sg])
			{
				displayStatePrev[led_part][dg][sg]=displayState[led_part][dg][sg];
				++diff_cnt;
			}
		}

		if(diff_cnt>0) mask|=(1<<dg);
	}

	if(control_ptr>=0xe000-1024)
	{
		printf("control_ptr #%4.4x\n",control_ptr);
	}

	if(!skip)
	{
	out_data[out_ptr++]=mask>>8;
	out_data[out_ptr++]=mask&255;

	control_inc(2);

	if(mask&0x4000) { out_data[out_ptr++]=pitch_id1; control_inc(1); }
	if(mask&0x2000) { out_data[out_ptr++]=pitch_id2; control_inc(1); }

	for(int dg=0;dg<9;++dg)
	{
		unsigned char b1=0;
		unsigned char b2=0;

		if(mask&(1<<dg))
		{
			b1|=(displayState[led_part][dg][SEG_A]&3);
			b1|=(displayState[led_part][dg][SEG_B]&3)<<2;
			b1|=(displayState[led_part][dg][SEG_C]&3)<<4;
			b1|=(displayState[led_part][dg][SEG_D]&3)<<6;

			b2|=(displayState[led_part][dg][SEG_E]&3);
			b2|=(displayState[led_part][dg][SEG_F]&3)<<2;
			b2|=(displayState[led_part][dg][SEG_G]&3)<<4;
			b2|=(displayState[led_part][dg][SEG_DP]&3)<<6;

			out_data[out_ptr++]=b1;
			out_data[out_ptr++]=b2;

			control_inc(2);
		}
	}
	}

	return true;
}



int convert_sequence(const unsigned char* seq,unsigned int seq_size,int led_part)
{
	int frames=0;

	unsigned int ptr=0;

	while(ptr<seq_size)
	{
		//re-encode frame

		for(int dp=0;dp<9;++dp)
		{
			displayState[led_part][dp][SEG_A]=seq[ptr+dp]&DSA?3:0;
			displayState[led_part][dp][SEG_B]=seq[ptr+dp]&DSB?3:0;
			displayState[led_part][dp][SEG_C]=seq[ptr+dp]&DSC?3:0;
			displayState[led_part][dp][SEG_D]=seq[ptr+dp]&DSD?3:0;
			displayState[led_part][dp][SEG_E]=seq[ptr+dp]&DSE?3:0;
			displayState[led_part][dp][SEG_F]=seq[ptr+dp]&DSF?3:0;
			displayState[led_part][dp][SEG_G]=seq[ptr+dp]&DSG?3:0;
			displayState[led_part][dp][SEG_DP]=seq[ptr+dp]&DSH?3:0;
		}

		int frame_repeat=seq[ptr+9];

		ptr+=10;

		for(int f=0;f<frame_repeat;++f)
		{
			//pack frame

			int mask=0;

			int pitch_id1=fetch_sound_frame();
			int pitch_id2=fetch_sound_frame();

			if(pitch_id1_prev!=pitch_id1)
			{
				pitch_id1_prev=pitch_id1;
				mask|=0x4000;
				++pitch_changes;
			}

			if(pitch_id2_prev!=pitch_id2)
			{
				pitch_id2_prev=pitch_id2;
				mask|=0x2000;
				++pitch_changes;
			}
			//printf("%2.2x %2.2x ",pitch_id1,pitch_id2);

			for(int dg=0;dg<9;++dg)
			{
				unsigned char b1=0;
				unsigned char b2=0;

				int diff_cnt=0;

				for(int sg=0;sg<8;++sg)
				{
					if(displayState[led_part][dg][sg]!=displayStatePrev[led_part][dg][sg])
					{
						displayStatePrev[led_part][dg][sg]=displayState[led_part][dg][sg];
						++diff_cnt;
					}
				}

				if(diff_cnt>0) mask|=(1<<dg);
			}

			out_data[out_ptr++]=mask>>8;
			out_data[out_ptr++]=mask&255;

			control_inc(2);

			if(mask&0x4000) { out_data[out_ptr++]=pitch_id1; control_inc(1); }
			if(mask&0x2000) { out_data[out_ptr++]=pitch_id2; control_inc(1); }

			for(int dg=0;dg<9;++dg)
			{
				unsigned char b1=0;
				unsigned char b2=0;

				if(mask&(1<<dg))
				{
					b1|=(displayState[led_part][dg][SEG_A]&3);
					b1|=(displayState[led_part][dg][SEG_B]&3)<<2;
					b1|=(displayState[led_part][dg][SEG_C]&3)<<4;
					b1|=(displayState[led_part][dg][SEG_D]&3)<<6;

					b2|=(displayState[led_part][dg][SEG_E]&3);
					b2|=(displayState[led_part][dg][SEG_F]&3)<<2;
					b2|=(displayState[led_part][dg][SEG_G]&3)<<4;
					b2|=(displayState[led_part][dg][SEG_DP]&3)<<6;

					out_data[out_ptr++]=b1;
					out_data[out_ptr++]=b2;

					control_inc(2);
				}
			}

			++control_frame;
		}
	}

	return frames;
}



int main(int argc, char* argv[])
{
	char filename[1024];

	int code_size=0x800;

	for(int led_part=0;led_part<4;++led_part)
	{
		memset(&BMP,0,sizeof(BMP));
		memset(displayState,0,sizeof(displayState));
		memset(displayStatePrev,0xff,sizeof(displayStatePrev));

		printf("part %i\n",led_part+1);

		sprintf(filename,"track_%i.txt",led_part+1);

		if(!parse_music_text(filename))
		{
			printf("no music file %s\n",filename);
			break;
		}

		out_ptr=0;
		control_ptr=CODE_AREA_SIZE;
		control_bank=0;
		control_frame=0;

		int frame=0;
		int skip_frames=-1;//4500-256;

		pitch_id1_prev=-1;
		pitch_id2_prev=-1;
		pitch_changes=0;

		frame+=convert_sequence(bad_aon_intro_seq,sizeof(bad_aon_intro_seq),led_part);

		while(1)
		{
			bool skip=false;
			if(skip_frames>=0&&frame<skip_frames) skip=true;

			if(!convert_frame(frame,led_part,skip)) break;

			++frame;
			++control_frame;
		}

		frame+=convert_sequence(bad_aon_end_seq,sizeof(bad_aon_end_seq),led_part);

		out_data[out_ptr++]=0x80;	//end of stream
		out_data[out_ptr++]=0x00;

		control_inc(2);

		printf("%i frames converted\n",frame);
		printf("%i pitch changes\n",pitch_changes);
		printf("%i bytes (%i max)\n",out_ptr,65536-code_size*2);

		sprintf(filename,"../z80/data_%i.bin",led_part+1);

		FILE* file=fopen(filename,"wb");

		if(file)
		{
			fwrite(out_data,out_ptr,1,file);

			for(int i=out_ptr;i<(65536-code_size*2);++i) fputc(0,file);	//pad to full size

			fclose(file);
		}

		sprintf(filename,"../z80/pitch_%i.bin",led_part+1);

		file=fopen(filename,"wb");

		if(file)
		{
			for(int i=0;i<pitch_table_len;++i)
			{
				unsigned char pair[2];
				pair[0]=pitch_table[i]&255;
				pair[1]=pitch_table[i]>>8;
				fwrite(pair,sizeof(pair),1,file);
			}
			fclose(file);
		}
	}

	printf("done\n");

	_getch();

	return 0;
}

