	device zxspectrum48
	
CODE_AREA_SIZE=#800		;how many bytes allocated for code duplication in banks 0 and 4
	
DSEG=#7f				;segments for current digit, also used for keypad matrix scanning

DSA=#01					;display segment names
DSB=#02
DSC=#04
DSD=#08
DSE=#10
DSF=#20
DSG=#40
DSH=#80

PIT_A=#a7				;8253 registers
PIT_B=#af
PIT_C=#b7
PIT_W=#bf

PIO_A=#c7				;8255 registers
PIO_B=#cf
PIO_C=#d7
PIO_W=#df

TIMER_CLK=1000000		;8253 clock frequency
TIMER_ISR=60*9*4/2		;timer interrupt rate (square mode)



;display common symbols

SYM_0=(DSA|DSB|DSC|DSD|DSE|DSF)
SYM_1=(DSB|DSC)
SYM_2=(DSA|DSB|DSD|DSE|DSG)
SYM_3=(DSA|DSB|DSC|DSD|DSG)
SYM_4=(DSB|DSC|DSF|DSG)
SYM_5=(DSA|DSC|DSD|DSF|DSG)
SYM_6=(DSA|DSC|DSD|DSE|DSF|DSG)
SYM_7=(DSA|DSB|DSC)
SYM_8=(DSA|DSB|DSC|DSD|DSE|DSF|DSG)
SYM_9=(DSA|DSB|DSC|DSD|DSF|DSG)

SYM_A=(DSA|DSB|DSC|DSE|DSF|DSG)
SYM_B=(DSC|DSD|DSE|DSF|DSG)
SYM_C=(DSA|DSD|DSE|DSF)
SYM_D=(DSB|DSC|DSD|DSE|DSG)
SYM_E=(DSA|DSD|DSE|DSF|DSG)
SYM_F=(DSA|DSE|DSF|DSG)
SYM_G=(DSA|DSC|DSD|DSE|DSF)
SYM_H=(DSB|DSC|DSE|DSF|DSG)
SYM_I=(DSE|DSF)
SYM_J=(DSB|DSC|DSD|DSE)
SYM_K=(DSB|DSC|DSE|DSF|DSG)
SYM_L=(DSD|DSE|DSF)
SYM_M=(DSA|DSC|DSE)
SYM_N=(DSA|DSB|DSC|DSE|DSF)
SYM_O=(DSA|DSB|DSC|DSD|DSE|DSF)
SYM_P=(DSA|DSB|DSE|DSF|DSG)
SYM_Q=(DSA|DSB|DSC|DSF|DSG)
SYM_R=(DSA|DSB|DSE|DSF)
SYM_S=(DSA|DSC|DSD|DSF|DSG)
SYM_T=(DSA|DSB|DSC)
SYM_U=(DSB|DSC|DSD|DSE|DSF)
SYM_V=(DSB|DSC|DSF)
SYM_W=(DSB|DSD|DSF)
SYM_X=(DSB|DSE|DSG)
SYM_Y=(DSB|DSC|DSD|DSF|DSG)
SYM_Z=(DSA|DSB|DSD|DSE|DSG)

	
	;there are eight 8K ROM banks in a 64K ROM
	;the ROM divided into two halves (0..3 and 4..7) that can be swapped with the PIO_A D6
	;the address space divided into 8K chunks as well:
	;0000 ROM bank 0 or 4
	;2000 RAM (at least 2K repeating inside the chunk, up to 8K in some models)
	;4000 ROM bank 1 or 5
	;6000 RAM mirror
	;8000 ROM bank 2 or 6
	;a000 RAM mirror
	;c000 ROM bank 3 or 7
	;e000 RAM mirror
	
	;animation and sound data spread across all banks, but with gaps at beginning of bank 0 and 4,
	;as it is easier to handle bank switching via duplication of all executable code in both banks


;--------------------------------------------------------------------

	org #0000
	
bank_0_begin:

	di
	
	ld sp,0

ram_clear:

	ld hl,#e000
	ld de,#e001
	ld bc,#2000-1
	ld (hl),l
	ldir
	
setup_pio:

	ld a,#82
	out (PIO_W),a		;ports A,C are outputs, B input, regular mode
	
	ld a,#80			;d0..3 led/keyb matrix select, d4 enable all sound, d6 select ROM page, d7 is always set for PIT gate ch2 (sound)
	ld (pio_a),a
	out (PIO_A),a
	ld a,#09			;both leds disabled at startup
	out (PIO_C),a		;d0 led1 (0=on,1=off), d3 led2
	ld a,#f0
	out (PIO_B),a

	jr start
	
	;there is a small gap in the ROM to insert the IM1 ISR trampoline to its fixed address
		
	org #0038
		
	jp timer_isr
	
start:

setup_timer_isr:

	ld a,#36			;channel 0 square wave lsb/msb binary
	out (PIT_W),a
	ld a,#b6			;channel 2 square wave lsb/msb binary
	out (PIT_W),a

	ld hl,TIMER_CLK/TIMER_ISR
	ld a,l
	out (PIT_A),a
	ld a,h
	out (PIT_A),a
	
	ld bc,0
	call pit_set_pitch

	;alternative registers reserved to the interrupt handler to save time,
	;as it runs at 2160 Hz
	;'B unused
	;'C display digit, runs in sync with E, 0..8
	;'DE display_view ptr, D stays the same, E runs 0..9*4-1
	;'HL pointer to display_sync

	ld c,#08
	ld de,display_view
	ld hl,display_sync
	exx

	;init data stream reader
	
	ld hl,CODE_AREA_SIZE
	;ld hl,#dc00
	ld (stream_ptr),hl
	xor a
	ld (stream_bank),a

	
	;interrupt handler refreshes display, everything else running in the main thread
	;thus the display needs double buffering
	
	im 1
	ei

start_delay:

	ld a,SYM_P
	ld (display_view+0*9+0),a
	ifdef PART_1
	ld a,SYM_1
	endif
	ifdef PART_2
	ld a,SYM_2
	endif
	ifdef PART_3
	ld a,SYM_3
	endif
	ifdef PART_4
	ld a,SYM_4
	endif
	ld (display_view+0*9+1),a
	
	ld bc,60*1
.wait1:
	call wait_60hz
	dec bc
	ld a,b
	or c
	jr nz,.wait1
	
	ld (display_view+0*9+0),a
	ld (display_view+0*9+1),a
	
	ld bc,60*3
.wait2:
	call wait_60hz
	dec bc
	ld a,b
	or c
	jr nz,.wait2

	
manual_sync:

	;check standard keyboard inputs, 1 on either bit 6 or 7 means a button is down
	;the outputs are driven by the LED multiplexer in the interrupt handler

	in a,(PIO_B)
	cpl
	and #c0
	;jr nz,manual_sync_skip
	jr z,manual_sync_skip
	
	ld a,SYM_S
	ld (display_view+0*9+0),a
	ld a,SYM_Y
	ld (display_view+0*9+1),a
	ld a,SYM_N
	ld (display_view+0*9+2),a
	ld a,SYM_C
	ld (display_view+0*9+3),a
	
	ld a,SYM_P
	ld (display_view+0*9+7),a
	ifdef PART_1
	ld a,SYM_1
	endif
	ifdef PART_2
	ld a,SYM_2
	endif
	ifdef PART_3
	ld a,SYM_3
	endif
	ifdef PART_4
	ld a,SYM_4
	endif
	ld (display_view+0*9+8),a

	jr manual_sync
	
manual_sync_skip:

	
main_loop:

	;animation frame rate is 30 fps, but music plays at 60 fps
	;so only decode a new frame once a couple frames displayed and played
	
	call wait_60hz
		
	ld a,(pitch_1)
	call pitch_from_table
	call pit_set_pitch
	
	;even frames are used to convert and update display data
	
	;decode frame data into segment data
		
	ld hl,frame_buf
	ld de,display_back
	ld b,9
	
.loop:

	call unpack_digit
	inc de
	djnz .loop
	
	;copy back buffer into the view buffer
	
	ld hl,display_back
	ld de,display_view
	ld bc,9*4
	ldir
	
	call wait_60hz
	
	ld a,(pitch_2)
	call pitch_from_table
	call pit_set_pitch

	
	
	;parse animation data
	
;data stream format is:
;
;for each 1/60 sec a variable number of bytes, at least two
;
; word update flags
;
; flags&8000 ? end of stream, mute sound and stop parsing
; flags&4000 ? sound pitch byte (first frame)
; flags&2000 ? sound pitch byte (second frame)
; flags&0001 ? digit 0, two bytes
; flags&0002 ? digit 1, two bytes
; flags&0004 ? digit 2, two bytes
; flags&0008 ? digit 3, two bytes
; flags&0010 ? digit 4, two bytes
; flags&0020 ? digit 5, two bytes
; flags&0040 ? digit 6, two bytes
; flags&0080 ? digit 7, two bytes
; flags&0100 ? digit 8, two bytes
;
; digit bytes are 2-bit values for each segment
; DDCCBBAA PPGGFFEE 
		
	call fetch_stream_byte		;flags msb
	ld b,a
	call fetch_stream_byte		;flags lsb
	ld c,a
	
	bit 7,b						;end marker
	jr z,.parse
	
	xor a
	ld (pitch_1),a
	ld (pitch_2),a
	
	jp end_loop
	
.parse:

	bit 6,b
	jr z,.noPitch1
	
	call fetch_stream_byte
	ld (pitch_1),a
	
.noPitch1:

	bit 5,b
	jr z,.noPitch2
	
	call fetch_stream_byte
	ld (pitch_2),a
	
.noPitch2:

	bit 0,c
	jr z,.noDigit0
	
	ld a,0
	call fetch_segment_data
	
.noDigit0:

	bit 1,c
	jr z,.noDigit1
	
	ld a,1
	call fetch_segment_data
	
.noDigit1:

	bit 2,c
	jr z,.noDigit2
	
	ld a,2
	call fetch_segment_data
	
.noDigit2:

	bit 3,c
	jr z,.noDigit3
	
	ld a,3
	call fetch_segment_data
	
.noDigit3:

	bit 4,c
	jr z,.noDigit4
	
	ld a,4
	call fetch_segment_data
	
.noDigit4:

	bit 5,c
	jr z,.noDigit5
	
	ld a,5
	call fetch_segment_data
	
.noDigit5:

	bit 6,c
	jr z,.noDigit6
	
	ld a,6
	call fetch_segment_data
	
.noDigit6:

	bit 7,c
	jr z,.noDigit7
	
	ld a,7
	call fetch_segment_data
	
.noDigit7:

	bit 0,b
	jr z,.noDigit8
	
	ld a,8
	call fetch_segment_data
	
.noDigit8:
	
	jp main_loop

	
	
end_loop:
	call wait_60hz
	jr end_loop



fetch_segment_data:

	add a,a
	ld hl,frame_buf
	add a,l
	ld l,a
	jr nc,$+3
	inc h
	
	call fetch_stream_byte
	
	ld (hl),a
	inc hl
	
	call fetch_stream_byte
	ld (hl),a
	
	ret
	
	
pitch_from_table:

	ld c,a
	ld b,0
	ld hl,pitch_table
	add hl,bc
	add hl,bc
	ld c,(hl)
	inc hl
	ld b,(hl)
	
	ret
	
	
	
;BC=0 mute sound, otherwise tone divider

pit_set_pitch:

	ld a,b
	or c
	jr z,.mute
	
	ld hl,(pit_div)
	ld a,h
	cp b
	jr nz,.divChange
	ld a,l
	cp c
	jr z,.divNoChange
	
.divChange:

	ld (pit_div),bc
	ld a,1
	ld (pit_update),a	;non-zero
	
.divNoChange:

	ld a,#10
	ld (sound_enable),a
	
	ld a,#00
	out (PIO_C),a

	ret
	
.mute:
	
	;ld a,0
	ld (sound_enable),a

	ld a,#09
	out (PIO_C),a
	
	ret
	
	
	
;in:  E,D,C,B planes 0..3, L=2-bit value, H=DSx macro (segment bit mask)
;out: E,D,C,B planes 0..3 modified

unpack_segment_planes:

	bit 1,l
	jr z,.upp0
	ld a,e
	or h
	ld e,a
	ld a,c
	or h
	ld c,a
	bit 0,l
	jr z,.upp1
	ld a,d
	or h
	ld d,a
	ld a,b
	or h
	ld b,a
	jp .upp1

.upp0:

	bit 0,l
	jr z,.upp1
	ld a,e
	or h
	ld e,a

.upp1:

	ret
	
	
	
;in:  HL=source data, points to two bytes, DE=display_back pointer (plane 0)
;out: writes to display_back directly, HL advanced by 2

unpack_digit:

	push bc
	push de
	
	ld (display_back_ptr),de
	
	ld de,0						;e=plane 0,d=plane 1
	ld bc,0						;c=plane 2,b=plane 3

	ld a,(hl)					;fetch first data byte %DDCCBBAA
	inc hl
	push hl
	push af
	
	and 3
	ld l,a
	ld h,DSA

	call unpack_segment_planes
	
	pop af
	rra
	rra
	push af
	and #3
	ld l,a
	ld h,DSB
	
	call unpack_segment_planes
	
	pop af
	rra
	rra
	push af
	and #3
	ld l,a
	ld h,DSC
	
	call unpack_segment_planes
	
	pop af
	rra
	rra
	and #3
	ld l,a
	ld h,DSD
	
	call unpack_segment_planes
	
	pop hl

	ld a,(hl)					;fetch second data byte #PPGGFFEE
	inc hl
	push hl
	push af
	
	and 3
	ld l,a
	ld h,DSE

	call unpack_segment_planes
	
	pop af
	rra
	rra
	push af
	and #3
	ld l,a
	ld h,DSF
	
	call unpack_segment_planes
	
	pop af
	rra
	rra
	push af
	and #3
	ld l,a
	ld h,DSG
	
	call unpack_segment_planes
	
	pop af
	rra
	rra
	and #3
	ld l,a
	ld h,DSH
	
	call unpack_segment_planes
	
	ld hl,(display_back_ptr)
	
	ld (hl),e
	ld a,l
	add a,9
	ld l,a
	ld (hl),d
	ld a,l
	add a,9
	ld l,a
	ld (hl),c
	ld a,l
	add a,9
	ld l,a
	ld (hl),b

	pop hl
	pop de
	pop bc
	
	ret
	
	
	
;64K ROM space has a sparse layout all across the 64K address space
;#0800..#1fff:0
;#4000..#5fff:0
;#8000..#9fff:0
;#c000..#dfff:0
;#0800..#1fff:1
;#4000..#5fff:1
;#8000..#9fff:1
;#c000..#dfff:1

fetch_stream_byte:

	push bc
	push de
	push hl
	
	;set bank, it remains always active
	
	ld hl,stream_bank
	ld a,(pio_a)
	or (hl)
	ld (pio_a),a			;interrupt handler also uses PIO_A actively
	out (PIO_A),a

	;fetch byte
	
	ld hl,(stream_ptr)
	ld c,(hl)
	
	;advance pointer considering the tricky layout
	
	inc hl
	
	ld a,h
	cp #20
	jr z,.skip_ram
	cp #60
	jr z,.skip_ram
	cp #a0
	jr z,.skip_ram
	cp #e0
	jr z,.skip_ram_and_code

.done:

	ld (stream_ptr),hl		;store modifed pointer, return byte
	ld a,c
	
	pop hl
	pop de
	pop bc
	
	ret

.skip_ram:

	add a,#20
	ld h,a
	jr .done
	
.skip_ram_and_code:

	add a,#20+(CODE_AREA_SIZE/256)
	ld h,a
	
	ld a,(stream_bank)				;change ROM bank
	xor #40
	ld (stream_bank),a
	
	jr .done
	
	
	
;timer interrupt only serves the display refresh with 4 brightness levels
;called 60*9*4 times per second (2160 Hz)

timer_isr:
	
	exa
	exx

	ld a,(pit_update)
	or a
	jr z,.noPITUpdate
	
	xor a
	ld (pit_update),a
	
	ld a,(pit_div+0)
	out (PIT_C),a
	ld a,(pit_div+1)
	out (PIT_C),a
	
.noPITUpdate:

	xor a					;clear segments before changing digit
	out (DSEG),a
	
	ld a,(sound_enable)
	ld b,a
	ld a,(pio_a)
	and #e0
	or b					;add sound mute bit
	or c					;C runs 0..8
	ld (pio_a),a
	out (PIO_A),a			;activate current digit, apply sound mute
	
	ld a,(de)				;display segments for current digit
	out (DSEG),a
	
	inc e
	ld a,e
	cp 9*4
	jr c,.skip1
	ld e,0
	inc (hl)				;mark full refresh cycle
.skip1:

	inc c
	ld a,c
	cp 9
	jr c,.skip2
	ld c,0
.skip2:

	exx
	exa

	ei
	ret						;<150t


	
wait_60hz:

	push af
	push hl
	
	ld hl,display_sync
	ld a,(hl)
	
.wait:

	cp (hl)
	jp z,.wait
	
	pop hl
	pop af

	ret
	
	
	


pitch_table:

	ifdef PART_1
	incbin "pitch_1.bin"
	endif
	ifdef PART_2
	incbin "pitch_2.bin"
	endif
	ifdef PART_3
	incbin "pitch_3.bin"
	endif
	ifdef PART_4
	incbin "pitch_4.bin"
	endif	


	display "code size: ",/d,$-bank_0_begin
	display "code top: ",/h,$
	
	

	org CODE_AREA_SIZE		;some first K allocated for code duplication in banks 0 and 4

bank_0_after_code:

data_stream:

DATA_STREAM_PREV=$
	ifdef PART_1
	incbin "data_1.bin",0,bank_0_begin+8192-$
	endif
	ifdef PART_2
	incbin "data_2.bin",0,bank_0_begin+8192-$
	endif
	ifdef PART_3
	incbin "data_3.bin",0,bank_0_begin+8192-$
	endif
	ifdef PART_4
	incbin "data_4.bin",0,bank_0_begin+8192-$
	endif
DATA_STREAM_OFF=$-DATA_STREAM_PREV

	display "bank 0: ",/d,$-bank_0_begin	;show current address, to have idea how much of ROM is used up
	
	ds bank_0_begin+8192-$,0				;pad ROM bank to 8K

bank_0_end:
	
	savebin "code.bin",bank_0_begin,CODE_AREA_SIZE-bank_0_begin	;save code to duplicate into bank 4
	savebin "bank_0.bin",bank_0_after_code,bank_0_end-bank_0_after_code
	
	
	
;--------------------------------------------------------------------

	org #4000
	
bank_1_begin:

DATA_STREAM_PREV=$
	ifdef PART_1
	incbin "data_1.bin",DATA_STREAM_OFF,bank_1_begin+8192-$
	endif
	ifdef PART_2
	incbin "data_2.bin",DATA_STREAM_OFF,bank_1_begin+8192-$
	endif
	ifdef PART_3
	incbin "data_3.bin",DATA_STREAM_OFF,bank_1_begin+8192-$
	endif
	ifdef PART_4
	incbin "data_4.bin",DATA_STREAM_OFF,bank_1_begin+8192-$
	endif
DATA_STREAM_OFF=DATA_STREAM_OFF+($-DATA_STREAM_PREV)

	display "bank 1: ",/d,$-bank_1_begin	;show current address, to have idea how much of ROM is used up
	
	ds bank_1_begin+8192-$,0				;pad ROM bank to 8K
	
bank_1_end:

	savebin "bank_1.bin",bank_1_begin,bank_1_end-bank_1_begin
	
	

;--------------------------------------------------------------------

	org #8000
	
bank_2_begin:

DATA_STREAM_PREV=$
	ifdef PART_1
	incbin "data_1.bin",DATA_STREAM_OFF,bank_2_begin+8192-$
	endif
	ifdef PART_2
	incbin "data_2.bin",DATA_STREAM_OFF,bank_2_begin+8192-$
	endif
	ifdef PART_3
	incbin "data_3.bin",DATA_STREAM_OFF,bank_2_begin+8192-$
	endif
	ifdef PART_4
	incbin "data_4.bin",DATA_STREAM_OFF,bank_2_begin+8192-$
	endif
DATA_STREAM_OFF=DATA_STREAM_OFF+($-DATA_STREAM_PREV)

	display "bank 2: ",/d,$-bank_2_begin	;show current address, to have idea how much of ROM is used up
	
	ds bank_2_begin+8192-$,0				;pad ROM bank to 8K
	
bank_2_end:

	savebin "bank_2.bin",bank_2_begin,bank_2_end-bank_2_begin



;--------------------------------------------------------------------

	org #c000
		
bank_3_begin:

DATA_STREAM_PREV=$
	ifdef PART_1
	incbin "data_1.bin",DATA_STREAM_OFF,bank_3_begin+8192-$
	endif
	ifdef PART_2
	incbin "data_2.bin",DATA_STREAM_OFF,bank_3_begin+8192-$
	endif
	ifdef PART_3
	incbin "data_3.bin",DATA_STREAM_OFF,bank_3_begin+8192-$
	endif
	ifdef PART_4
	incbin "data_4.bin",DATA_STREAM_OFF,bank_3_begin+8192-$
	endif
DATA_STREAM_OFF=DATA_STREAM_OFF+($-DATA_STREAM_PREV)

	display "bank 3: ",/d,$-bank_3_begin	;show current address, to have idea how much of ROM is used up
	
	ds bank_3_begin+8192-$,0				;pad ROM bank to 8K
	
bank_3_end:

	savebin "bank_3.bin",bank_3_begin,bank_3_end-bank_3_begin

;--------------------------------------------------------------------

	org #0000

bank_4_begin:

	org CODE_AREA_SIZE

bank_4_after_code:

DATA_STREAM_PREV=$
	ifdef PART_1
	incbin "data_1.bin",DATA_STREAM_OFF,bank_4_begin+8192-$
	endif
	ifdef PART_2
	incbin "data_2.bin",DATA_STREAM_OFF,bank_4_begin+8192-$
	endif
	ifdef PART_3
	incbin "data_3.bin",DATA_STREAM_OFF,bank_4_begin+8192-$
	endif
	ifdef PART_4
	incbin "data_4.bin",DATA_STREAM_OFF,bank_4_begin+8192-$
	endif
DATA_STREAM_OFF=DATA_STREAM_OFF+($-DATA_STREAM_PREV)

	display "bank 4: ",/d,$-bank_4_begin	;show current address, to have idea how much of ROM is used up
	
	ds bank_4_begin+8192-$,0				;pad ROM bank to 8K
	
bank_4_end:

	savebin "bank_4.bin",bank_4_after_code,bank_4_end-bank_4_after_code



;--------------------------------------------------------------------

	org #4000
		
bank_5_begin:

DATA_STREAM_PREV=$
	ifdef PART_1
	incbin "data_1.bin",DATA_STREAM_OFF,bank_5_begin+8192-$
	endif
	ifdef PART_2
	incbin "data_2.bin",DATA_STREAM_OFF,bank_5_begin+8192-$
	endif
	ifdef PART_3
	incbin "data_3.bin",DATA_STREAM_OFF,bank_5_begin+8192-$
	endif
	ifdef PART_4
	incbin "data_4.bin",DATA_STREAM_OFF,bank_5_begin+8192-$
	endif
DATA_STREAM_OFF=DATA_STREAM_OFF+($-DATA_STREAM_PREV)

	display "bank 5: ",/d,$-bank_5_begin	;show current address, to have idea how much of ROM is used up
	
	ds bank_5_begin+8192-$,0				;pad ROM bank to 8K
	
bank_5_end:

	savebin "bank_5.bin",bank_5_begin,bank_5_end-bank_5_begin



;--------------------------------------------------------------------

	org #8000
		
bank_6_begin:

DATA_STREAM_PREV=$
	ifdef PART_1
	incbin "data_1.bin",DATA_STREAM_OFF,bank_6_begin+8192-$
	endif
	ifdef PART_2
	incbin "data_2.bin",DATA_STREAM_OFF,bank_6_begin+8192-$
	endif
	ifdef PART_3
	incbin "data_3.bin",DATA_STREAM_OFF,bank_6_begin+8192-$
	endif
	ifdef PART_4
	incbin "data_4.bin",DATA_STREAM_OFF,bank_6_begin+8192-$
	endif
DATA_STREAM_OFF=DATA_STREAM_OFF+($-DATA_STREAM_PREV)

	display "bank 6: ",/d,$-bank_6_begin	;show current address, to have idea how much of ROM is used up
	
	ds bank_6_begin+8192-$,0				;pad ROM bank to 8K
	
bank_6_end:

	savebin "bank_6.bin",bank_6_begin,bank_6_end-bank_6_begin
	


;--------------------------------------------------------------------

	org #c000
		
bank_7_begin:

DATA_STREAM_PREV=$
	ifdef PART_1
	incbin "data_1.bin",DATA_STREAM_OFF,bank_7_begin+8192-$
	endif
	ifdef PART_2
	incbin "data_2.bin",DATA_STREAM_OFF,bank_7_begin+8192-$
	endif
	ifdef PART_3
	incbin "data_3.bin",DATA_STREAM_OFF,bank_7_begin+8192-$
	endif
	ifdef PART_4
	incbin "data_4.bin",DATA_STREAM_OFF,bank_7_begin+8192-$
	endif
DATA_STREAM_OFF=DATA_STREAM_OFF+($-DATA_STREAM_PREV)

	display "bank 7: ",/d,$-bank_7_begin	;show current address, to have idea how much of ROM is used up
	
	ds bank_7_begin+8192-$,0				;pad ROM bank to 8K
	
bank_7_end:

	savebin "bank_7.bin",bank_7_begin,bank_7_end-bank_7_begin
	


;--------------------------------------------------------------------

	org #e000			;RAM
	
	align 256

display_view:			ds 9*4,0	;must not cross 256 byte page
display_back:			ds 9*4,0	;back buffer
display_sync:			db 0
pio_a:					db 0

display_back_ptr:		dw 0

stream_ptr:				dw 0
stream_bank:			db 0

pitch_1:				db 0
pitch_2:				db 0
pit_div:				dw 0
pit_div_prev:			dw 0
pit_update:				db 0
sound_enable:			db 0

	org #e100
frame_buf:				ds 9*2,0